==> usocket-udp_2.5.tar.gz: usocket-udp-2.5/usocket-cmucl.lisp <==
;;;; -*- Mode: Lisp -*-
;;;; $Id: usocket-cmucl.lisp 592 2008-10-28 12:10:59Z binghe $

;;;; Note: this code only support CMUCL after 2008-8 snapshot (not 19e)
;;;; WHEN you use LOCAL-HOST/LOCAL-PORT!

(in-package :usocket)

(defmethod socket-close :after ((socket datagram-usocket))
  (setf (%open-p socket) nil))

(defun socket-connect/udp (host port &key element-type timeout deadline nodelay
			   (local-host nil local-host-p)
			   (local-port nil local-port-p)
			   &aux
			   (local-bind-p (fboundp 'ext::bind-inet-socket)))
  (declare (ignore element-type timeout deadline nodelay))
  (when (and local-host-p (not local-bind-p))
     (unsupported 'local-host 'socket-connect :minimum "Snapshot 2008-08 (19E)"))
  (when (and local-port-p (not local-bind-p))
     (unsupported 'local-port 'socket-connect :minimum "Snapshot 2008-08 (19E)"))

  (let ((socket))
    (setf socket
	  (if (and host port)
	      (let ((args (list (host-to-hbo host) port :datagram)))
		(when (and local-bind-p (or local-host-p local-port-p))
		  (nconc args (list :local-host (when local-host
						  (host-to-hbo local-host))
				    :local-port local-port)))
		(with-mapped-conditions (socket)
		  (apply #'ext:connect-to-inet-socket args)))
	      (if (or local-host-p local-port-p)
		  (with-mapped-conditions (socket)
		    (apply #'ext:create-inet-listener
			   (nconc (list (or local-port 0) :datagram)
				  (when (and local-host-p
					     (ip/= local-host *wildcard-host*))
				    (list :host (host-to-hbo local-host))))))
		  (with-mapped-conditions (socket)
		    (ext:create-inet-socket :datagram)))))
    (if socket
	(let ((usocket (make-datagram-socket socket)))
	  (ext:finalize usocket #'(lambda () (when (%open-p usocket)
					       (ext:close-socket socket))))
	  usocket)
	(let ((err (unix:unix-errno)))
	  (when err (cmucl-map-socket-error err))))))

(defmethod socket-send ((usocket datagram-usocket) buffer length &key host port)
  (with-mapped-conditions (usocket)
    (ext:inet-sendto (socket usocket) buffer length (if host (host-to-hbo host)) port)))

(defmethod socket-receive ((usocket datagram-usocket) buffer length)
  (let ((real-buffer (or buffer
                         (make-array length :element-type '(unsigned-byte 8))))
        (real-length (or length
                         (length buffer))))
    (multiple-value-bind (nbytes remote-host remote-port)
        (with-mapped-conditions (usocket)
          (ext:inet-recvfrom (socket usocket) real-buffer real-length))
      (when (plusp nbytes)
        (values real-buffer nbytes remote-host remote-port)))))

==> usocket-udp_2.6.tar.gz: usocket-udp-2.6/usocket-cmucl.lisp <==
;;;; -*- Mode: Lisp -*-
;;;; $Id: usocket-cmucl.lisp 822 2010-09-24 10:24:44Z binghe $

;;;; Note: this code only support CMUCL after 2008-8 snapshot (not 19e)
;;;; WHEN you use LOCAL-HOST/LOCAL-PORT!

(in-package :usocket)

(defmethod socket-close :after ((socket datagram-usocket))
  (setf (%open-p socket) nil))

(defun socket-connect/udp (host port &key element-type timeout deadline nodelay
			   (local-host nil local-host-p)
			   (local-port nil local-port-p)
			   &aux
			   (local-bind-p (fboundp 'ext::bind-inet-socket)))
  (declare (ignore element-type timeout deadline nodelay))
  (when (and local-host-p (not local-bind-p))
     (unsupported 'local-host 'socket-connect :minimum "Snapshot 2008-08 (19E)"))
  (when (and local-port-p (not local-bind-p))
     (unsupported 'local-port 'socket-connect :minimum "Snapshot 2008-08 (19E)"))

  (let ((socket))
    (setf socket
	  (if (and host port)
	      (let ((args (list (host-to-hbo host) port :datagram)))
		(when (and local-bind-p (or local-host-p local-port-p))
		  (nconc args (list :local-host (when local-host
						  (host-to-hbo local-host))
				    :local-port local-port)))
		(with-mapped-conditions (socket)
		  (apply #'ext:connect-to-inet-socket args)))
	      (if (or local-host-p local-port-p)
		  (with-mapped-conditions (socket)
		    (apply #'ext:create-inet-listener
			   (nconc (list (or local-port 0) :datagram)
				  (when (and local-host-p
					     (ip/= local-host *wildcard-host*))
				    (list :host (host-to-hbo local-host))))))
		  (with-mapped-conditions (socket)
		    (ext:create-inet-socket :datagram)))))
    (if socket
	(let ((usocket (make-datagram-socket socket)))
	  (ext:finalize usocket #'(lambda () (when (%open-p usocket)
					       (ext:close-socket socket))))
	  usocket)
	(let ((err (unix:unix-errno)))
	  (when err (cmucl-map-socket-error err))))))

(defmethod socket-send ((usocket datagram-usocket) buffer length &key host port)
  (with-mapped-conditions (usocket)
    (ext:inet-sendto (socket usocket) buffer length (if host (host-to-hbo host)) port)))

(defmethod socket-receive ((usocket datagram-usocket) buffer length &key)
  (let ((real-buffer (or buffer
                         (make-array length :element-type '(unsigned-byte 8))))
        (real-length (or length
                         (length buffer))))
    (multiple-value-bind (nbytes remote-host remote-port)
        (with-mapped-conditions (usocket)
          (ext:inet-recvfrom (socket usocket) real-buffer real-length))
      (when (plusp nbytes)
        (values real-buffer nbytes remote-host remote-port)))))
